from argparse import Namespace
from logging import debug, info
from re import search
from time import strftime
from typing import List, Tuple

from pyerge import utils, TMPLOGFILE, TMERGE_LOGFILE, DEVNULL


def emerge(arguments: List[str], build=True) -> Tuple[bytes, bytes]:
    """
    Run emerge command.

    :param arguments:
    :param build:
    :return:
    """
    info(f"running emerge with: {' '.join(arguments)}")
    cmd = f"sudo /usr/bin/emerge --nospinner {' '.join(arguments)}"
    if build:
        return_code, stderr = utils.run_cmd(cmd, use_system=True)
        debug(f'RC: {return_code.decode("utf-8")}, Errors: {stderr.decode("utf-8")}')
        return return_code, stderr
    output, stderr = utils.run_cmd(cmd)
    return output, stderr


# <=><=><=><=><=><=><=><=><=><=><=><=> chk_upd <=><=><=><=><=><=><=><=><=><=><=><=>
def check_upd(local_chk: bool) -> None:
    """
    Check system updates.

    :param local_chk:
    """
    utils.delete_content(TMPLOGFILE)
    utils.delete_content(TMERGE_LOGFILE)
    with open(file=TMPLOGFILE, mode='w', encoding='utf-8') as tmp, open(file=TMERGE_LOGFILE, mode='w', encoding='utf-8') as log:
        tmp.write(strftime('%a %b %d %H:%M:%S %Z %Y') + '\n')
        if not local_chk:
            info('Start syncing portage...')
            debug(f'sudo eix-sync >> {TMPLOGFILE} > {DEVNULL}')
            utils.run_cmd(f'sudo eix-sync >> {TMPLOGFILE} > {DEVNULL}', use_system=True)
        info('Checking updates...')
        output, error = emerge('-pvNDu --color n --with-bdeps=y @world'.split(), build=False)
        debug(f'Error: {error}')  # type: ignore
        log.write(output.decode('utf-8'))
        log.write(error.decode('utf-8'))

    info('Creating log file...')
    debug(f'cat {TMERGE_LOGFILE} >> {TMPLOGFILE}')
    utils.run_cmd(f'cat {TMERGE_LOGFILE} >> {TMPLOGFILE}', use_system=True)
    debug(f'cat {TMERGE_LOGFILE} | genlop -pn >> {TMPLOGFILE}')
    utils.run_cmd(f'cat {TMERGE_LOGFILE} | genlop -pn >> {TMPLOGFILE}', use_system=True)


# <=><=><=><=><=><=><=><=><=><=><=><=> tmerge <=><=><=><=><=><=><=><=><=><=><=><=>
def post_emerge(args: List[str], return_code: bytes) -> None:
    """
    Run actions after emerge.

    :param args:
    :param return_code:
    """
    pretend, world = check_emerge_opts(args)
    if not int(return_code) and not pretend and world:
        info('Clearing emerge log')
        with open(file=TMPLOGFILE, mode='w', encoding='utf-8'), open(file=TMERGE_LOGFILE, mode='w', encoding='utf-8') as log:
            log.write('Total: 0 packages, Size of downloads: 0 KiB')


def deep_clean(args: List[str], opts: Namespace, return_code: bytes) -> None:
    """
    Run deep clean after emerge.

    :param args:
    :param opts:
    :param return_code:
    """
    pretend, world = check_emerge_opts(args)
    if not int(return_code) and not pretend and world:
        output, error = emerge(['-pc'], build=False)
        info('Deep clean')
        info(f'Output details:{output.decode("utf-8")}')
        debug(f'Errors details:{error.decode("utf-8")}')
        deep_run(opts, output)


def deep_run(opts: Namespace, output: bytes):
    """
    Run deep clean emegre without gent0o sources.

    :param opts:
    :param output:
    """
    if opts.deep_run:
        regexp = search(r'All selected packages:\s(.*)\n', output.decode('utf-8'))
        if regexp is not None:
            package_list = [package for package in regexp.group(1).split(' ') if 'gentoo-sources' not in package]
            if package_list:
                debug(f'Cleaning {len(package_list)} packages')
                emerge(['-c'] + package_list, build=True)
            else:
                info('Nothing to clean')
                debug(f'All packages: {regexp.group(1)}')


def check_emerge_opts(args: List[str]) -> Tuple[bool, bool]:
    """
    Check options in emerge command.

    :param args:
    :return:
    """
    return bool('pretend' in ' '.join(args)), bool('world' in ' '.join(args))


def is_portage_running() -> bool:
    """
    Check if potrage command in currently running.

    :return: True if it is running, False otherwise
    """
    out, _ = utils.run_cmd('pgrep -f /usr/bin/emerge')
    return bool(out)


def run_emerge(emerge_opts: List[str], opts: Namespace) -> None:
    """
    Run update of system.

    :param emerge_opts: list of arguments for emege
    :param opts: cli arguments
    """
    if opts.action == 'emerge' and opts.online:
        ret_code, _ = emerge(emerge_opts, build=True)
        post_emerge(emerge_opts, ret_code)
        if opts.deep_print or opts.deep_run:
            deep_clean(emerge_opts, opts, ret_code)


def run_check(opts: Namespace) -> None:
    """
    Run checking system updates.

    :param opts: cli arguments
    """
    if opts.action == 'check' and (opts.online or opts.local):
        check_upd(opts.local)



from unittest.mock import patch

from pytest import mark


@mark.parametrize('return_code, pretend, world', [(b'0', False, False), (b'0', True, False),
                                                  (b'0', True, True), (b'1', False, False),
                                                  (b'1', False, True), (b'1', True, False),
                                                  (b'1', True, True)])
def test_deep_clean_not_run(return_code, pretend, world):
    from argparse import Namespace
    with patch('pyerge.tmerge.check_emerge_opts') as check_emerge_opts_mock, \
            patch('pyerge.tmerge.emerge') as emerge_mock, \
            patch('pyerge.tmerge.deep_run') as deep_run_mock:
        from pyerge import tmerge
        check_emerge_opts_mock.return_value = (pretend, world)
        tmerge.deep_clean([''], Namespace(), return_code)
        emerge_mock.assert_not_called()
        deep_run_mock.assert_not_called()


def test_deep_clean_run():
    from argparse import Namespace
    with patch('pyerge.tmerge.check_emerge_opts') as check_emerge_opts_mock, \
            patch('pyerge.tmerge.emerge') as emerge_mock, \
            patch('pyerge.tmerge.deep_run') as deep_run_mock:
        from pyerge import tmerge
        opts = Namespace()
        output_and_rc = b'0'
        check_emerge_opts_mock.return_value = (False, True)
        emerge_mock.return_value = (output_and_rc, output_and_rc)
        tmerge.deep_clean([''], opts, output_and_rc)
        emerge_mock.assert_called_once_with(['-pc'], build=False)
        deep_run_mock.assert_called_once_with(opts, output_and_rc)


def test_deep_run_not_selected():
    from argparse import Namespace
    with patch('pyerge.tmerge.emerge') as emerge_mock:
        from pyerge import tmerge
        tmerge.deep_run(opts=Namespace(deep_run=False), output=b'')
        emerge_mock.assert_not_called()


def test_deep_run_wrong_output():
    from argparse import Namespace
    with patch('pyerge.tmerge.emerge') as emerge_mock:
        from pyerge import tmerge
        tmerge.deep_run(opts=Namespace(deep_run=True), output=b'All selected packages: ')
        emerge_mock.assert_not_called()


def test_deep_run_output_with_two_packages():
    from argparse import Namespace
    with patch('pyerge.tmerge.emerge') as emerge_mock:
        from pyerge import tmerge
        output = b'\n\nAll selected packages: =sys-kernel/gentoo-sources-5.10.76-r1 =dev-python/hypothesis-6.27.1\n\n'
        opts = Namespace(deep_run=True)
        tmerge.deep_run(opts=opts, output=output)
        emerge_mock.assert_called_once_with(['-c', '=dev-python/hypothesis-6.27.1'], build=True)


def test_deep_run_output_with_only_gentoo():
    from argparse import Namespace
    with patch('pyerge.tmerge.emerge') as emerge_mock:
        from pyerge import tmerge
        output = b'\n\nAll selected packages: =sys-kernel/gentoo-sources-5.10.76-r1\n\n'
        opts = Namespace(deep_run=True)
        tmerge.deep_run(opts=opts, output=output)
        emerge_mock.assert_not_called()


@mark.parametrize('list_str, result', [(['--pretend', '--verbose', '--newuse', '--deep', '--update', '@world'], (True, True)),
                                       (['--pretend', '--verbose', 'conky'], (True, False)),
                                       (['-f', 'conky'], (False, False)),
                                       (['', '@world'], (False, True)),
                                       (['', 'conky'], (False, False))])
def test_check_emerge_opts(list_str, result):
    from pyerge import tmerge
    assert tmerge.check_emerge_opts(list_str) == result


def test_is_portage_running():
    from pyerge import tmerge
    with patch('pyerge.tmerge.utils') as utils_mock:
        utils_mock.run_cmd.return_value = (b'3456\n', b'')
        assert tmerge.is_portage_running() is True
        utils_mock.run_cmd.assert_called_once_with('pgrep -f /usr/bin/emerge')


def test_is_portage_not_running():
    from pyerge import tmerge
    with patch('pyerge.tmerge.utils') as utils_mock:
        utils_mock.run_cmd.return_value = (b'', b'')
        assert tmerge.is_portage_running() is False
        utils_mock.run_cmd.assert_called_once_with('pgrep -f /usr/bin/emerge')


def test_run_emerge():
    from argparse import Namespace
    with patch('pyerge.tmerge.emerge') as emerge_mock, \
            patch('pyerge.tmerge.post_emerge') as post_emerge_mock, \
            patch('pyerge.tmerge.deep_clean') as deep_clean:
        ret_code = b'0'
        emerge_mock.return_value = (ret_code, b'')
        from pyerge import tmerge
        emerge_opts = ['-NDu', '@world']
        opts = Namespace(action='emerge', online=True, deep_print=True)
        tmerge.run_emerge(emerge_opts=emerge_opts, opts=opts)
        post_emerge_mock.assert_called_once_with(emerge_opts, ret_code)
        deep_clean.assert_called_once_with(emerge_opts, opts, ret_code)


def test_run_check():
    from argparse import Namespace
    with patch('pyerge.tmerge.check_upd') as check_upd_mock:
        from pyerge import tmerge
        opts = Namespace(action='check', local=True, online=True)
        tmerge.run_check(opts)
        check_upd_mock.assert_called_once_with(opts.local)



from unittest import mock

from pytest import mark


PORT_TMP_DIR = '/var/tmp/portage'


@mark.parametrize('size, result', [('12K', 12), ('34567k', 34567), ('31M', 31744), ('233m', 238592),
                                   ('2G', 2097152), ('1g', 1048576), ('500', 500), ('450.1', 450)])
def test_convert2blocks(size, result):
    from pyerge import utils
    assert utils.convert2blocks(size) == result


def test_tmpfs_not_mounted():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'/dev/sda2 on /boot type ext2 (rw,noatime,errors=continue,user_xattr,acl)\n' \
                                    b'rpc_pipefs on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw,relatime)\n' \
                                    b'none on /run/user/1000 type tmpfs (rw,relatime,mode=700,uid=1000)\n', b''
        assert utils.is_tmpfs_mounted() is False


def test_tmpfs_mounted():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'/dev/sda2 on /boot type ext2 (rw,noatime,errors=continue,user_xattr,acl)\n' \
                                    b'rpc_pipefs on /var/lib/nfs/rpc_pipefs type rpc_pipefs (rw,relatime)\n' \
                                    b'none on /run/user/1000 type tmpfs (rw,relatime,mode=700,uid=1000)\n' \
                                    b'tmpfs on /var/tmp/portage type tmpfs (rw,relatime,size=1000k,nr_inodes=1048576)\n', b''
        assert utils.is_tmpfs_mounted() is True


def test_unmounttmpfs(opt_emerge_nonlocal_with_1g):
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        utils.unmounttmpfs(opt_emerge_nonlocal_with_1g)
        run_cmd_mock.assert_called_once_with(f'sudo umount -f {PORT_TMP_DIR}')


def test_mounttmpfs():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        utils.mounttmpfs(size='2G')
        run_cmd_mock.assert_called_once_with(f'sudo mount -t tmpfs -o size=2G,nr_inodes=1M tmpfs {PORT_TMP_DIR}')


def test_remounttmpfs():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        utils.remounttmpfs(size='2G')
        run_cmd_mock.assert_has_calls([mock.call.run_cmd(f'sudo umount -f {PORT_TMP_DIR}'),
                                       mock.call.run_cmd(f'sudo mount -t tmpfs -o size=2G,nr_inodes=1M tmpfs {PORT_TMP_DIR}')])


def test_size_of_not_mounted_tmpfs():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'Filesystem      1K-blocks     Used  Available Use% Mounted on\n' \
                                    b'/dev/sda2          126931    76647      43731  64% /boot\n' \
                                    b'none              4043868        0    4043868   0% /run/user/1000\n' \
                                    b'tmpfs                1000        0       1000   0% /var/tmp/portage\n', b''
        assert utils.size_of_mounted_tmpfs() == 1000


def test_size_of_mounted_tmpfs():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'Filesystem      1K-blocks     Used  Available Use% Mounted on\n' \
                                    b'/dev/sda2          126931    76647      43731  64% /boot\n' \
                                    b'none              4043868        0    4043868   0% /run/user/1000\n', b''
        assert utils.size_of_mounted_tmpfs() == 0


def test_is_internet_connected():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'PING 89.16.167.134 (89.16.167.134) 56(84) bytes of data.\n' \
                                    b'64 bytes from 89.16.167.134: icmp_seq=1 ttl=47 time=52.2 ms\n\n' \
                                    b'--- 89.16.167.134 ping statistics ---\n' \
                                    b'1 packets transmitted, 1 received, 0% packet loss, time 0ms\n' \
                                    b'rtt min/avg/max/mdev = 52.212/52.212/52.212/0.000 ms\n', b''
        assert utils.is_internet_connected() is True


def test_is_internet_not_connected():
    from pyerge import utils
    with mock.patch('pyerge.utils.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'', b''
        assert utils.is_internet_connected() is False


def test_delete_content():
    from pyerge import utils
    with mock.patch('pyerge.utils.open') as open_mock:
        utils.delete_content(fname='/tmp/emerge.log')
        open_mock.assert_called_once_with(file='/tmp/emerge.log', mode='w', encoding='utf-8')


def test_run_cmd_as_subprocess_ver1():
    from pyerge import utils
    with mock.patch('pyerge.utils.Popen') as popen_mock:
        process_mock = mock.Mock()
        out = b'Filesystem      1K-blocks     Used  Available Use% Mounted on\n' \
              b'/dev/sda2          126931    76647      43731  64% /boot\n'
        err = b''
        attrs = {'communicate.return_value': (out, err)}
        process_mock.configure_mock(**attrs)
        popen_mock.return_value = process_mock
        assert utils.run_cmd(cmd='df') == (out, err)
        popen_mock.assert_called_once_with(['df'], stderr=-1, stdout=-1)


def test_run_cmd_as_subprocess_ver2():
    from pyerge import utils
    from subprocess import Popen
    with mock.patch.object(Popen, 'communicate') as communicate_mock:
        out = b'Filesystem      1K-blocks     Used  Available Use% Mounted on\n' \
              b'/dev/sda2          126931    76647      43731  64% /boot\n'
        err = b''
        communicate_mock.return_value = (out, err)
        assert utils.run_cmd(cmd='df') == (out, err)
        communicate_mock.assert_called_once_with()


def test_run_cmd_as_sysyem():
    from pyerge import utils
    with mock.patch('pyerge.utils.system') as system_mock:
        system_mock.return_value = 0
        assert utils.run_cmd(cmd='df', use_system=True) == (b'0', b'')
        system_mock.assert_called_once_with('df')


def test_set_portage_tmpdir(monkeypatch):
    from os import environ
    from pyerge import utils
    monkeypatch.setitem(environ, 'PORTAGE_TMPDIR', '')
    assert utils.set_portage_tmpdir() == '/var/tmp/portage'


def test_portage_tmpdir_already_set(monkeypatch):
    from os import environ
    from pyerge import utils
    monkeypatch.setitem(environ, 'PORTAGE_TMPDIR', 'some_value')
    assert utils.set_portage_tmpdir() == 'some_value'


def test_handling_mounting_mount(opt_emerge_nonlocal_with_1g):
    from pyerge import utils
    with mock.patch('pyerge.utils.mounttmpfs') as mounttmpfs_mock:
        with mock.patch('pyerge.utils.is_tmpfs_mounted') as is_tmpfs_mounted_mock:
            is_tmpfs_mounted_mock.return_value = False
            mounttmpfs_mock.return_value = None
            utils.handling_mounting(opt_emerge_nonlocal_with_1g)


def test_handling_mounting_remounte(opt_emerge_nonlocal_with_1g):
    from pyerge import utils
    with mock.patch('pyerge.utils.size_of_mounted_tmpfs') as size_of_mounted_tmpfs_mock:
        with mock.patch('pyerge.utils.convert2blocks') as convert2blocks_mock:
            with mock.patch('pyerge.utils.is_tmpfs_mounted') as is_tmpfs_mounted_mock:
                is_tmpfs_mounted_mock.return_value = True
                size_of_mounted_tmpfs_mock.return_value = 2
                convert2blocks_mock.return_value = 1
                utils.handling_mounting(opt_emerge_nonlocal_with_1g)


def test_handling_mounting_else(opt_emerge_nonlocal_with_1g):
    from pyerge import utils
    with mock.patch('pyerge.utils.size_of_mounted_tmpfs') as size_of_mounted_tmpfs_mock:
        with mock.patch('pyerge.utils.convert2blocks') as convert2blocks_mock:
            with mock.patch('pyerge.utils.is_tmpfs_mounted') as is_tmpfs_mounted_mock:
                is_tmpfs_mounted_mock.return_value = True
                size_of_mounted_tmpfs_mock.return_value = 1
                convert2blocks_mock.return_value = 1
                utils.handling_mounting(opt_emerge_nonlocal_with_1g)



import sys
from argparse import ArgumentParser, Namespace
from logging import basicConfig, DEBUG, INFO, ERROR, info, error
from typing import List

from pyerge import tmerge, utils, __version__


def run_parser() -> None:
    """
    Function to collect command line arguments.

    Construct main object with correct set of parameters.
    """
    parser = ArgumentParser(description='Emerge in temporary RAM disk')
    parser.add_argument('-s', '--size', action='store', dest='size', default='4G', help='Size or RAM disk, default 4G')
    parser.add_argument('-l', '--check_local', action='store_true', dest='local', default=False, help='check locally')
    parser.add_argument('-d', '--clean-print', action='store_true', dest='deep_print', default=False, help='print deep clean info after emerge')
    parser.add_argument('-c', '--clean-run', action='store_true', dest='deep_run', default=False, help='run deep clean after emerge')
    parser.add_argument('-w', '--world', action='store_true', dest='world', default=False, help='run emerge -NDu @world')
    parser.add_argument('-r', '--pretend_world', action='store_true', dest='pretend_world', default=False, help='run emerge -pvNDu @world')
    parser.add_argument('-q', '--quiet', action='store_true', dest='quiet', default=False, help='no output from pyerge itself only form other tools')
    parser.add_argument('-v', '--verbose', action='count', dest='verbose', default=0, help='Increase output verbosity')
    parser.add_argument('-V', '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('action', help='check or emerge')
    opts, emerge_opts = parser.parse_known_args()
    level = DEBUG if opts.verbose else INFO
    if opts.quiet:
        level = ERROR
    basicConfig(format='%(asctime)s | %(levelname)-6s | %(message)s', level=level)
    if opts.action not in ['check', 'emerge']:
        error(f'Wrong options: {opts} {emerge_opts}')
        sys.exit(1)
    main_exec(opts, emerge_opts)


def main_exec(opts: Namespace, emerge_opts: List[str]) -> None:
    """
    Main execution function.

    :param opts: cli arguments
    :param emerge_opts: list of arguments for emege
    """
    if opts.world:
        emerge_opts = ['--with-bdeps=y', '--keep-going=y', '--newuse', '--deep', '--update', '@world']
    if opts.pretend_world:
        emerge_opts = ['--with-bdeps=y', '--pretend', '--verbose', '--newuse', '--deep', '--update', '@world']
    info(f'Pyerge version: {__version__}')
    opts.online = utils.is_internet_connected()

    if not tmerge.is_portage_running():
        utils.set_portage_tmpdir()
        utils.handling_mounting(opts)
        tmerge.run_emerge(emerge_opts, opts)
        tmerge.run_check(opts)
        utils.unmounttmpfs(opts)
    else:
        info('emerge already running!')



import io
from os.path import abspath, dirname, join

from setuptools import setup, find_packages

from pyerge import __version__

here = abspath(dirname(__file__))

with io.open(join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

with io.open(join(here, 'requirements.txt'), encoding='utf-8') as f:
    requires = f.read().splitlines()

setup(
    name='pyerge',
    version=__version__,
    description='It is Python wrapper tool for emerge (Gentoo package manager - Portage). It can mount RAM disk of defined size and compile packages inside it',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/emcek/pyerge',
    author='Michal Plichta',
    license='MIT License',
    entry_points={'console_scripts': ['pye = pyerge.cli:run_parser',
                                      'e_sync = pyerge.tools:e_sync',
                                      'e_dl = pyerge.tools:e_dl',
                                      'e_curr = pyerge.tools:e_curr',
                                      'e_eut = pyerge.tools:e_eut',
                                      'e_eta = pyerge.tools:e_eta',
                                      'e_log = pyerge.tools:e_log',
                                      'e_sta = pyerge.tools:e_sta',
                                      'e_prog = pyerge.tools:e_prog',
                                      'e_upd = pyerge.tools:e_upd',
                                      'e_raid = pyerge.tools:run_e_raid',
                                      'glsa = pyerge.glsa:run_glsa']},
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Environment :: Console',
                 'Environment :: X11 Applications',
                 'License :: OSI Approved :: MIT License',
                 'Operating System :: POSIX',
                 'Operating System :: Unix',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 'Programming Language :: Python :: 3.8',
                 'Programming Language :: Python :: 3.9',
                 'Programming Language :: Python :: 3.10',
                 'Topic :: Desktop Environment',
                 'Topic :: Software Development',
                 'Topic :: Software Development :: Libraries :: Python Modules',
                 'Topic :: System :: Monitoring',
                 'Topic :: Utilities'],
    keywords='gentoo portage emerge conky linux',
    packages=find_packages(exclude=['tests']),
    install_requires=requires,
    include_package_data=True,
    python_requires='>=3.6',
    extras_require={'testing': ['pytest']},
    project_urls={'Bug Reports': 'https://github.com/emcek/pyerge/issues',
                  'Source': 'https://github.com/emcek/pyerge'},
)



from argparse import ArgumentParser
from re import match
from sys import modules
from typing import List
from urllib import request, error

from bs4 import BeautifulSoup

from pyerge import utils


def glsa_list(elements: int) -> str:
    """
    List GLSAs with number and name as string with new lines.

    :param elements: number of elements
    :return: string with new lines
    """
    return '\n'.join(_rss(regex=r'GLSA\s(\d{6}-\d{2}:\s.*)', elements=elements))


def glsa_test(elements: int) -> str:
    """
    Test system against GLSAs.

    :param elements: number of elements
    :return: string with result
    """
    glsalist = ' '.join(_rss(regex=r'GLSA\s(\d{6}-\d{2}):\s.*', elements=elements))
    out, err = utils.run_cmd(f'glsa-check -t {glsalist}')
    if err == b'This system is not affected by any of the listed GLSAs\n':
        return 'System is not affected by any of listed GLSAs'
    return out.decode('utf-8').strip().replace('\n', ',')


def _rss(regex: str, elements: int) -> List[str]:
    """
    Parse web page and find all matching tags.

    :param regex: regural expresion
    :param elements: number of elements to return
    :return: list of strings
    """
    with request.urlopen('https://security.gentoo.org/glsa/feed.rss1') as rss_page:  # nosec
        rss_html = rss_page.read().decode('utf-8')
    all_versions = _collect_all_maching_entries(rss_html, regex)
    return all_versions[0:elements]


def _collect_all_maching_entries(html: str, regex: str) -> List[str]:
    """
    Parse web page and find all matching tags.

    :param html: web page
    :param regex: regural expresion
    :return: list of strings
    """
    tmp_list = []
    soup = BeautifulSoup(html, 'html.parser')
    all_a_tags = soup.find_all('title')
    for tag in all_a_tags:
        found = match(regex, tag.text)
        if found:
            tmp_list.append(found.group(1))
    return tmp_list


def run_glsa():
    """Run GLSA module to test or to list."""
    parser = ArgumentParser(description='Check and list GLSA easly')
    parser.add_argument('action', help='list or test')
    parser.add_argument('-e', '--elements', action='store', dest='elements', type=int, default='5', help='number of elements')
    args = parser.parse_args()
    try:
        print(getattr(modules['pyerge.glsa'], f'glsa_{args.action}')(args.elements))
    except (AttributeError, KeyError, error.HTTPError) as err:
        print(f'{err}')
